#!/bin/bash
if [ $UID -ne 0 ]; then
	echo "Root Privilege Required"
	exit
fi
shopt -s extglob

INDEV=0
IPSSEND=`dirname $0`/ipssend

function get_drivever {
	${IPSSEND} drivever $1 $2 $3| (
	while read LL; do
		if [ "${LL%%*( ):*}" == "Vendor" ]; then
			echo -n " vendor='${LL##*: }'"
		elif [ "${LL%%*( ):*}" == "Revision Level" ]; then
			echo -n " rev='${LL##*: }'"
		elif [ "${LL%%*( ):*}" == "Serial Number" ]; then
			echo -n " serial='${LL##*: }'"
		fi
	done
	)
}
function do_controller {
	if [ "${LINE%%*( ):*}" == "Controller type" ]; then
		echo -n " type='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Controlling BIOS version" ]; then
		echo -n " bios='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Firmware version" ]; then
		echo -n " firmware='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Device driver version" ]; then
		echo -n " driver_version='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Controller slot information" ]; then
		echo -n " slot='${LINE##*:*( )}'"
	fi
}

function do_logical {
	if [ "${LINE%%*( ):*}" == "Status of logical drive" ]; then
		echo -n " status='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "RAID level" ]; then
		echo -n " raidlevel='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Size (in MB)" ]; then
		echo -n " size='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Number of chunks" ]; then
		echo -n " chunks='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Stripe-unit size" ]; then
		echo -n " stripesize='${LINE##*:*( )}'"
	elif [ "${LINE%%*( ):*}" == "Part of array" ]; then
		echo -n " array='${LINE##*:*( )}'"
	elif [[ "$LINE" == Array*stripe*order* ]]; then
		ANAME=${LINE##Array }
		ANAME=${ANAME%% stripe*}
		echo "/>"
		echo -n "<array name='$ANAME' member_devs='${LINE##*:*( )}'"
	fi


}
function do_physical {
	if [ "${LINE%%?:*( )}" == "Channel #" ]; then
		CNUM=${LINE##*#}
		CNUM=${CNUM%%:}
		if [ $INDEV -ne 0 ]; then
			echo "/>"
		fi
		if [ $CNUM -ne 1 ]; then
			echo "</channel>"
		fi
		echo "<channel num='$CNUM'>"
		INDEV=0
	elif [ "${LINE%%*( )+(0|1|2|3|4|5|6|7|8|9)}" == "Initiator at SCSI ID" ]; then
		if [ $INDEV -ne 0 ]; then 
			echo "/>"
		fi
		echo "<initiator id='${LINE##*ID }'></initiator>"
	elif [ "${LINE%%*( )+(0|1|2|3|4|5|6|7|8|9)}" == "Target on SCSI ID" ]; then
		if [ $INDEV -ne 0 ]; then 
			echo "/>"
		fi
		echo -n "<target id='${LINE##*ID }'"
		get_drivever $CONTROLLER $CNUM ${LINE##*ID }
		INDEV=1
	elif [[ "${LINE}" == Device\ is\ a\ * ]]; then
		echo -n " type='${LINE##*is a }'"
	elif [ "${LINE%%*( ):*}" == "PFA (Yes/No)" ]; then
		echo -n " pfa='${LINE##*: }'"
	elif [ "${LINE%%*( ):*}" == "State" ]; then
		echo -n " state='${LINE##*: }'"
	elif [ "${LINE%%*( ):*}" == "FRU part number" ]; then
		echo -n " fru='${LINE##: }'"
	elif [ "${LINE%%*( ):*}" == "Size (in MB)/(in sectors)" ]; then
		echo -n " size='${LINE##*: }'"
	elif [ "${LINE%%*( ):*}" == "Device ID" ]; then
		echo -n " devid='${LINE##*: }'"
	fi
}

function check_state {
	if [ "$LINE" == "Controller information" ]; then
		if [ "$STATE" != "null" ]; then
			echo "/>"
		fi
		export STATE=controller
		return 0
	fi
	if [ "${LINE%%*( )+(1|2|3|4|5|6|7|8|9|0)}" == "Logical drive number" ]; then
		if [ "$STATE" != "null" ]; then
			echo "/>"
		fi
		echo -n "<logical_drive num='${LINE##*ber*( )}'"
		export STATE=logical
		return 0
	fi
	if [ "$LINE" == "Physical device information" ]; then
		if [ "$STATE" != "null" ]; then
			echo "/>"
		fi
		export STATE=physical
		return 0
	fi
	return 1
}
			
	
if [ -d /proc/scsi/ips ]; then # ServeRAID information
	echo -n "<serveraid controlling_bios='"
	BVER=`${IPSSEND} getversion|grep "lling BIOS"`
	echo ${BVER##*: }"'>"
	for i in /proc/scsi/ips/*; do
		CONTROLLER=${i##*/}
		CONTROLLER=$(($CONTROLLER + 1))
		STATE=null
		echo -n "<controller num='$CONTROLLER'"
		${IPSSEND} getconfig 1| (
		DNUM=-1
		while read LINE; do 
			if check_state; then
				continue
			fi
			if [ "$STATE" == "controller" ]; then
				do_controller
				continue
			fi
			if [ "$STATE" == "logical" ]; then
				do_logical
				continue
			fi
			if [ "$STATE" == "physical" ]; then
				do_physical
				continue
			fi
		done)
		echo "></serverraid>"
		echo "></channel>"
		echo "></serveraid>"
	done
fi
