#!/usr/bin/env python
import sys
import gtk
from xml.dom import minidom
import string
import gobject
import time
import re
			
def get_man_dev(manid,devid):
	pciids = open("pci.ids")
	man = None
	dev= None
	for i in pciids:
		if man is None:
			if re.match("%04x" % manid,i):
				man=string.strip(re.split("  ",i)[1])
		else:
			if re.match("\t%04x" % devid,i):
				dev=string.strip(re.split("  ",i)[1])
	return (man,dev)

def write_row(model,tag,value="",parent=None):
	iter=model.insert_before(parent,None)
	model.set_value(iter,0,tag);
	model.set_value(iter,1,value);
	return iter

def fill_model(model,doc):
	for survey in doc.getElementsByTagName("survey"):
		iter = model.insert_after(None,None)
		#model.set_value(iter,0,time.strftime("%D",time.localtime(int(survey.getAttribute("timestamp")))))
		model.set_value(iter,0,time.ctime(int(survey.getAttribute("timestamp"))))
		model.set_value(iter,1,"")
		model.set_value(iter,2,survey)
		for system in survey.getElementsByTagName("system"):
			siter=model.insert_after(iter,None)
			model.set_value(siter,0,system.getAttribute("name"))
			model.set_value(siter,1,system.getAttribute("status"))
			model.set_value(siter,2,system)
			
		
def disp_sys(node,model):
	for distro in node.getElementsByTagName("distro"):
		par=write_row(model,"OS")
		write_row(model,"Version",distro.getAttribute("release"),par)
		for kernel in node.getElementsByTagName("kernel"):
			write_row(model,"Kernel Version",kernel.getAttribute("version"),par)
			write_row(model,"Kernel Arguments",kernel.getAttribute("cmdline"),par)
		dpar=write_row(model,"Modules","",par)
		mods=node.getElementsByTagName("modules")[0]
		for mod in string.split(mods.getAttribute("loaded"),"||"):
			write_row(model,mod,"",dpar)
			
	for sysinfo in node.getElementsByTagName("sysinfo"):
		par=write_row(model,"System Information")
		write_row(model,"Vendor",sysinfo.getAttribute("vendor"),par)
		write_row(model,"Product",sysinfo.getAttribute("product"),par)
		write_row(model,"Version",sysinfo.getAttribute("version"),par)
		write_row(model,"UUID",sysinfo.getAttribute("uuid"),par)
		write_row(model,"Serial Number",sysinfo.getAttribute("serial"),par)
	for bios in node.getElementsByTagName("bios"):
		par=write_row(model,"BIOS Information")
		write_row(model,"Vendor",bios.getAttribute("vendor"),par)
		write_row(model,"Version",bios.getAttribute("version"),par)
		write_row(model,"Date",bios.getAttribute("date"),par)
		write_row(model,"Address","0x%X" % int(bios.getAttribute("address")),par)
	if len(node.getElementsByTagName("sysconf")) > 0:
		syscparent=write_row(model,"Jumpers/DIP Switches/etc")
	for sysconf in node.getElementsByTagName("sysconf"):
			for str in string.split(sysconf.getAttribute("options"),"|"):
				write_row(model,"",str,syscparent)
			#write_row(model,"Option Setting/Descritpion",sysconf.getAttribute("options"),syscparent)
	for oeminfo in node.getElementsByTagName("oeminfo"):
		par=write_row(model,"OEM Specific Information")
		for str in string.split(oeminfo.getAttribute("info"),"|"):
			write_row(model,"",str,par)
	if len(node.getElementsByTagName("processor")) > 1:
        	procparent=write_row(model,"Processors")
	else:
		procparent=None
		
	for proc in node.getElementsByTagName("processor"):
		if procparent is None:
			par=write_row(model,"Processor")
			write_row(model,"Socket",proc.getAttribute("socket"),par)
		else:
			par=write_row(model,proc.getAttribute("socket"),"",procparent)
		write_row(model,"Type",proc.getAttribute("type"),par)
		write_row(model,"Vendor",proc.getAttribute("vendor"),par)
		write_row(model,"Family",proc.getAttribute("family"),par)
		write_row(model,"Version",proc.getAttribute("version"),par)
		write_row(model,"UUID",proc.getAttribute("uuid"),par)
		write_row(model,"Misc",proc.getAttribute("miscinf"),par)
		write_row(model,"Flags",proc.getAttribute("flags"),par)
		write_row(model,"Upgrade Type",proc.getAttribute("upgrade"),par)
		write_row(model,"Voltage",proc.getAttribute("voltage"),par)
		write_row(model,"Bus Clock",proc.getAttribute("busspeed"),par)
		write_row(model,"Current Clock",proc.getAttribute("currspeed"),par)
		write_row(model,"Max Clock",proc.getAttribute("maxspeed"),par)
		write_row(model,"Status",proc.getAttribute("status"),par)
	for memc in node.getElementsByTagName("memcontroller"):
		par=write_row(model,"Memory Controller")
		write_row(model,"Purpose",memc.getAttribute("purpose"),par)
		write_row(model,"Location",memc.getAttribute("location"),par)
		write_row(model,"Max Size",memc.getAttribute("maxsize"),par)
		write_row(model,"EC Capabilities",memc.getAttribute("ec_type"),par)
		write_row(model,"Slots",memc.getAttribute("slots"),par)
		#Could show last mem controller as parent of all memory, but is bad idea because 
		#no *actual* relationship is guaranteed by anything in the DMI spec
		#npar=par
	if len(node.getElementsByTagName("memory")) > 1:
		mempar=write_row(model,"Memory Modules")
	else:
		mempar=None
	for mem in node.getElementsByTagName("memory"):
		if mempar is None:
			par=write_row(model,"Memory Module")
			write_row(model,"Name",mem.getAttribute("name"),par)
		else:
			par=write_row(model,mem.getAttribute("name"),"",mempar)
		write_row(model,"Packaging",mem.getAttribute("packaging"),par)
		write_row(model,"Group",mem.getAttribute("group"),par)
		write_row(model,"Location",mem.getAttribute("location"),par)
		write_row(model,"Characteristics",mem.getAttribute("characteristics"),par)
		write_row(model,"Size",mem.getAttribute("size"),par)
		write_row(model,"Type",mem.getAttribute("type"),par)
		write_row(model,"Bus Width",mem.getAttribute("bus_width"),par)
		write_row(model,"Data Width",mem.getAttribute("data_width"),par)
		write_row(model,"Array Handle",mem.getAttribute("array_handle"),par)
	if len(node.getElementsByTagName("slot")) > 1:
		spar=write_row(model,"Slots")
	else:
		spar=None
	for slot in node.getElementsByTagName("slot"):
		if spar is None:
			par=write_row(model,"Slot Information")
			write_row(model,"Name",slot.getAttribute("name"))
		else:
			par=write_row(model,slot.getAttribute("name"),"",spar)
		write_row(model,"Type",slot.getAttribute("slot_type"),par)
		write_row(model,"Bus Width",slot.getAttribute("bus_width"),par)
		write_row(model,"Length",slot.getAttribute("length"),par)
		write_row(model,"Physical Slot",slot.getAttribute("phys_slot"),par)
		if slot.getAttribute("populated") != "0":
			write_row(model,"In Use","Yes",par)
			(man,dev)=get_man_dev(int(slot.getAttribute("man_id")),int(slot.getAttribute("dev_id")))
			write_row(model,"Manufacturer ID","0x%04X (%s)" % (int(slot.getAttribute("man_id")),man),par)
			write_row(model,"Device ID","0x%04X (%s)" % (int(slot.getAttribute("dev_id")),dev),par)
			if slot.attributes.get("driver") is not None:
				write_row(model,"Driver",slot.getAttribute("driver"),par)
			else:
				write_row(model,"Driver","Not Detected",par)
				
		else:
			write_row(model,"In Use","No",par)
	if len(node.getElementsByTagName("scsidev")) > 1:
		npar=write_row(model,"SCSI Devices")
	for scsi in node.getElementsByTagName("scsidev"):
		par=write_row(model,scsi.getAttribute("host")+","+scsi.getAttribute("channel")+","+scsi.getAttribute("id"),"",npar)
		write_row(model,"Vendor",scsi.getAttribute("vendor"),par)
		write_row(model,"Model",scsi.getAttribute("model"),par)
		write_row(model,"Rev",scsi.getAttribute("rev"),par)
		write_row(model,"Type",scsi.getAttribute("type"),par)
		write_row(model,"ANSI SCSI Rev",scsi.getAttribute("ansi_scsi_rev"),par)

	for sraid in node.getElementsByTagName("serveraid"):
		srpar=write_row(model,"ServeRAID")
		for srcon in sraid.getElementsByTagName("controller"):
			scpar=write_row(model,"Controller %s" % srcon.getAttribute("num"),"",srpar)
	if len(node.getElementsByTagName("net")) > 1:
		npar=write_row(model,"Network Devices")
	else:
		npar=None
	for net in node.getElementsByTagName("net"):
		if npar is None:
			par=write_row(model,"Network Device")
			write_row(model,"Name",net.getAttribute("device"))
		else:
			par=write_row(model,net.getAttribute("device"),"",npar)
		if (net.getAttribute("up") != "0"):
			write_row(model,"Status","Up",par)
			write_row(model,"IP Address",net.getAttribute("ipaddr"),par)
			write_row(model,"Broadcast",net.getAttribute("broadcast"),par)
			write_row(model,"Netmask",net.getAttribute("netmask"),par)
		else:
			write_row(model,"Status","Down",par)
		write_row(model,"Received Packets",net.getAttribute("rxpackets"),par)
		write_row(model,"Errored received packets",net.getAttribute("rxerr"),par)
		write_row(model,"Overruns on receipt",net.getAttribute("rxoverruns"),par)
		write_row(model,"Received Packets Dropped",net.getAttribute("rxdrop"),par)
		write_row(model,"Transmitted Packets",net.getAttribute("txpackets"),par)
		write_row(model,"Errored transmitted packets",net.getAttribute("txerr"),par)
		write_row(model,"Overruns on transmit",net.getAttribute("txoverruns"),par)
		write_row(model,"Transmit Packets Dropped",net.getAttribute("txdrop"),par)
		write_row(model,"Collisions",net.getAttribute("collisions"),par)
	if len(node.getElementsByTagName("mount")) is not 0:
		mpar=write_row(model,"Mounts")
	for mount in node.getElementsByTagName("mount"):
		write_row(model,"",mount.getAttribute("device")+" mounted on "+mount.getAttribute("mountpoint")+" as "+mount.getAttribute("type")+" with options "+mount.getAttribute("options"),mpar)
		
		

		

		
		
		
		
		
		
		
def show_system(sel, viewmod):
	(model, iter) = sel.get_selected()
	if iter is not None:
		node = model.get_value(iter,2)
		viewmod.clear()
		if node.nodeName == "system":
			disp_sys(node,viewmod)
	
	
	
	

			
doc = minidom.parse("cluster.xml")
window = gtk.Window(gtk.WINDOW_TOPLEVEL)
window.resize(700,480)
window.set_title("Hardware Inventories")
navscroll=gtk.ScrolledWindow()
viewscroll=gtk.ScrolledWindow()
window.connect("destroy",gtk.mainquit)
window.set_border_width(5)

navmod = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_PYOBJECT)
viewmod = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
fill_model(navmod,doc)
navview = gtk.TreeView(navmod)
viewview = gtk.TreeView(viewmod)
navview.set_size_request(150,0)
navview.get_selection().connect("changed",show_system,viewmod)
navview.set_headers_visible(1)
viewtag = gtk.TreeViewColumn("Tag",gtk.CellRendererText(),text=0)
viewtag.set_resizable(1)
viewval = gtk.TreeViewColumn("Value",gtk.CellRendererText(),text=1)
viewval.set_resizable(1)
viewview.append_column(viewtag)
viewview.append_column(viewval)
navcol = gtk.TreeViewColumn("Date/System",gtk.CellRendererText(),text=0)
navcol.set_resizable(1)
statcol = gtk.TreeViewColumn("Status",gtk.CellRendererText(),text=1)
statcol.set_resizable(1)
navview.append_column(navcol)
navview.append_column(statcol)
hpane=gtk.HPaned()
window.add(hpane)
navscroll.add(navview)
viewscroll.add(viewview)
hpane.add1(navscroll)
hpane.add2(viewscroll)
window.show_all()
gtk.main()
